@@ -23,8 +23,8 @@
 #include <math.h>
 #include <yaml.h>
 
-/* Tweaks files use C locale so make sure we don't accidentally use locale-aware functions. */
-#pragma GCC poison strtof strtod strtold
+#define MS_TWEAKS_NO_LOCALE_FORMATTING
+#include "ms-tweaks-files-poison.h"
 
 /* Undocumented default from settingstree.py in postmarketos-tweaks. */
 #define CONF_TWEAKS_DEFAULT_WEIGHT 50
@@ -421,7 +421,6 @@ static void
 merge_weights (int *into, const int from)
 {
   g_assert (into);
-  g_assert (from);
 
   /* Only overwrite the weight if it wasn't specified. We assume that even if the default was
    * specified, the original definition didn't care about the weight. */
