@@ -11,6 +11,8 @@
 #include "ms-tweaks-preferences-page.h"
 
 #include "backends/ms-tweaks-backend-gsettings.h"
+#include "backends/ms-tweaks-backend-gtk3settings.h"
+#include "backends/ms-tweaks-backend-symlink.h"
 #include "backends/ms-tweaks-backend-xresources.h"
 #include "ms-tweaks-backend-interface.h"
 #include "ms-tweaks-callback-handlers.h"
@@ -167,9 +169,7 @@ setting_data_to_color_widget (const MsTweaksSetting *setting_data,
 
 
 static void
-file_widget_open_file_picker (GtkButton                             *widget,
-                              GdkEvent                              *evnt,
-                              MsTweaksPreferencesPageFilePickerMeta *metadata)
+file_widget_open_file_picker (GtkButton *widget, MsTweaksPreferencesPageFilePickerMeta *metadata)
 {
   GtkFileDialog *file_picker_dialog = gtk_file_dialog_new ();
 
@@ -185,18 +185,27 @@ static const char *none_selected_label = "(None selected)";
 
 
 static void
-file_widget_unset (GtkButton                             *widget,
-                   GdkEvent                              *evnt,
-                   MsTweaksPreferencesPageFilePickerMeta *metadata)
+file_widget_unset (GtkButton *widget, MsTweaksPreferencesPageFilePickerMeta *metadata)
 {
-  MsTweaksBackendInterface *backend = MS_TWEAKS_BACKEND_GET_IFACE (metadata->backend_state);
+  MsTweaksBackendInterface *backend;
   GError *error = NULL;
+  gboolean success;
+
+  g_assert (metadata);
+  g_assert (metadata->backend_state);
+  g_assert (GTK_IS_LABEL (metadata->file_picker_label));
+  g_assert (MS_IS_TWEAKS_BACKEND (metadata->backend_state));
+
+  backend = MS_TWEAKS_BACKEND_GET_IFACE (metadata->backend_state);
 
   g_assert (backend->set_value);
 
-  backend->set_value (metadata->backend_state, NULL, &error);
+  success = backend->set_value (metadata->backend_state, NULL, &error);
 
-  gtk_label_set_label (GTK_LABEL (metadata->file_picker_label), none_selected_label);
+  if (success)
+    gtk_label_set_label (GTK_LABEL (metadata->file_picker_label), none_selected_label);
+  else
+    ms_tweaks_callback_handlers_show_error_toast (metadata->toast_overlay, error->message);
 }
 
 
@@ -225,6 +234,7 @@ setting_data_to_file_widget (const MsTweaksSetting                 *setting_data
   adw_action_row_add_suffix (ADW_ACTION_ROW (file_picker_row), reset_selection_button);
   adw_action_row_add_suffix (ADW_ACTION_ROW (file_picker_row), file_picker_button);
 
+  gtk_box_append (GTK_BOX (file_picker_box), metadata->file_picker_label);
   gtk_box_append (GTK_BOX (file_picker_box), file_picker_icon);
   gtk_button_set_child (GTK_BUTTON (file_picker_button), file_picker_box);
   gtk_button_set_icon_name (GTK_BUTTON (reset_selection_button), "document-revert-symbolic");
@@ -390,14 +400,18 @@ ms_tweaks_preferences_page_initable_init (GInitable     *initable,
       case MS_TWEAKS_BACKEND_IDENTIFIER_GSETTINGS:
         backend_state = ms_tweaks_backend_gsettings_new (setting_data);
         break;
+      case MS_TWEAKS_BACKEND_IDENTIFIER_GTK3SETTINGS:
+        backend_state = ms_tweaks_backend_gtk3settings_new (setting_data);
+        break;
+      case MS_TWEAKS_BACKEND_IDENTIFIER_SYMLINK:
+        backend_state = ms_tweaks_backend_symlink_new (setting_data);
+        break;
       case MS_TWEAKS_BACKEND_IDENTIFIER_XRESOURCES:
         backend_state = ms_tweaks_backend_xresources_new (setting_data);
         break;
       case MS_TWEAKS_BACKEND_IDENTIFIER_CSS:
-      case MS_TWEAKS_BACKEND_IDENTIFIER_GTK3SETTINGS:
       case MS_TWEAKS_BACKEND_IDENTIFIER_SYSFS:
       case MS_TWEAKS_BACKEND_IDENTIFIER_SOUNDTHEME:
-      case MS_TWEAKS_BACKEND_IDENTIFIER_SYMLINK:
       default:
         ms_tweaks_debug (setting_data->name,
                          "Unimplemented backend type '%s'",
@@ -417,8 +431,19 @@ ms_tweaks_preferences_page_initable_init (GInitable     *initable,
         widget_value = MS_TWEAKS_BACKEND_GET_IFACE (backend_state)->get_value (backend_state);
 
       /* Handle mappings. */
-      if (widget_value)
-        ms_tweaks_mappings_handle_get (widget_value, setting_data);
+      if (widget_value) {
+        g_autoptr (GError) mapping_error = NULL;
+        gboolean success;
+
+        success = ms_tweaks_mappings_handle_get (widget_value, setting_data, &mapping_error);
+
+        if (!success) {
+          ms_tweaks_warning (setting_data->name,
+                             "Failed to handle mappings, ignoring: %s",
+                             mapping_error->message);
+          continue;
+        }
+      }
 
       if (setting_widget_is_valid) {
         MsTweaksCallbackMeta *callback_meta;
