@@ -17,9 +17,9 @@
 #include "ms-feedback-panel.h"
 #include "ms-plugin-panel.h"
 
-#include "pmos-tweaks/ms-tweaks-page-builder.h"
-#include "pmos-tweaks/ms-tweaks-parser.h"
-#include "pmos-tweaks/ms-tweaks-utils.h"
+#include "conf-tweaks/ms-tweaks-parser.h"
+#include "conf-tweaks/ms-tweaks-preferences-page.h"
+#include "conf-tweaks/ms-tweaks-utils.h"
 
 #include <glib/gi18n.h>
 
@@ -104,21 +104,34 @@ add_ms_tweaks_page (gpointer value, gpointer user_data)
 
   if (page_widget) {
     GtkStackPage *stack_page = gtk_stack_add_titled (self->stack,
-                                                     GTK_WIDGET (page_widget), page_data->name,
-                                                     page_data->name);
+                                                     GTK_WIDGET (page_widget), page_data->name_i18n,
+                                                     page_data->name_i18n);
     /* TODO: Read icon from base64 property of settings definitions. */
-    gtk_stack_page_set_icon_name (stack_page, "applications-science-symbolic");
+    gtk_stack_page_set_icon_name (stack_page, "conf-tweaks-symbolic");
   }
 }
 
 
+static void
+do_toggle_conf_tweaks (GSettings *settings, char *key, gpointer user_data)
+{
+  MobileSettingsWindow *self = MOBILE_SETTINGS_WINDOW (user_data);
+  gboolean conf_tweaks_enabled = g_settings_get_boolean (settings, key);
+
+  /* Flip! */
+  conf_tweaks_enabled = !conf_tweaks_enabled;
+
+  ms_panel_switcher_refilter (self->panel_switcher,
+                              conf_tweaks_enabled ? GTK_FILTER_CHANGE_LESS_STRICT : GTK_FILTER_CHANGE_MORE_STRICT);
+}
+
+
 static void
 ms_settings_window_constructed (GObject *object)
 {
   MobileSettingsWindow *self = MOBILE_SETTINGS_WINDOW (object);
   MobileSettingsApplication *app = MOBILE_SETTINGS_APPLICATION (g_application_get_default ());
   GtkWidget *device_panel;
-  GList *pages_sorted_by_weight = NULL;
   GHashTable *parser_page_table = NULL;
 
   G_OBJECT_CLASS (mobile_settings_window_parent_class)->constructed (object);
@@ -137,10 +150,19 @@ ms_settings_window_constructed (GObject *object)
     }
   }
 
+  ms_tweaks_parser_parse_definition_files (self->ms_tweaks_parser, TWEAKS_DATA_DIR);
   parser_page_table = ms_tweaks_parser_get_page_table (self->ms_tweaks_parser);
-  pages_sorted_by_weight = ms_tweaks_parser_sort_by_weight (parser_page_table);
 
-  g_list_foreach (pages_sorted_by_weight, add_ms_tweaks_page, self);
+  if (g_hash_table_size (parser_page_table) != 0) {
+    GList *pages_sorted_by_weight = ms_tweaks_parser_sort_by_weight (parser_page_table);
+    g_autoptr (GAction) toggle_conf_tweaks;
+
+    g_list_foreach (pages_sorted_by_weight, add_ms_tweaks_page, self);
+
+    toggle_conf_tweaks = g_settings_create_action (self->settings, "enable-conf-tweaks");
+    g_signal_connect (self->settings, "changed::enable-conf-tweaks", G_CALLBACK (do_toggle_conf_tweaks), self);
+    g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (toggle_conf_tweaks));
+  }
 }
 
 
