@@ -20,7 +20,6 @@
 #define TOUCH_MAPPING_PATH_PREFIX "/org/gnome/desktop/peripherals/touchscreens/"
 #define TOUCH_MAPPING_KEY "output"
 
-
 typedef struct {
   char *name;
 
@@ -34,21 +33,20 @@ typedef struct {
 
 
 struct _MsConvergencePanel {
-  AdwBin     parent;
+  MsPanel              parent;
 
-  const MsDock *dock;
+  const MsDock        *dock;
   AdwPreferencesGroup *dock_pref_group;
-  GtkStack  *dock_stack;
-  GSettings *touch_settings;
-  GtkListBox *docks_listbox;
-  AdwActionRow *map_touch_screen_row;
-  GtkWidget *map_touch_screen_switch;
+  GtkStack            *dock_stack;
+  GSettings           *touch_settings;
+  GtkListBox          *docks_listbox;
+  AdwActionRow        *map_touch_screen_row;
+  GtkWidget           *map_touch_screen_switch;
 
-
-  MsHeadTracker *tracker;
+  MsHeadTracker       *tracker;
 };
 
-G_DEFINE_TYPE (MsConvergencePanel, ms_convergence_panel, ADW_TYPE_BIN)
+G_DEFINE_TYPE (MsConvergencePanel, ms_convergence_panel, MS_TYPE_PANEL)
 
 
 /*
@@ -109,7 +107,7 @@ ms_dock_has_touch (const MsDock *dock)
 static const MsDock *
 find_dock (MsHead *head)
 {
-  for (int i = 0; i < G_N_ELEMENTS (docks); i++) {
+  for (guint i = 0; i < G_N_ELEMENTS (docks); i++) {
     if ((STR_IS_NULL_OR_EMPTY (docks[i].make) || g_strcmp0 (docks[i].make, head->make) == 0) &&
         (STR_IS_NULL_OR_EMPTY (docks[i].model) || g_strcmp0 (docks[i].model, head->model) == 0) &&
         (STR_IS_NULL_OR_EMPTY (docks[i].serial) || g_strcmp0 (docks[i].serial, head->serial_number) == 0)) {
@@ -249,7 +247,7 @@ on_head_tracker_changed (MsConvergencePanel *self, GParamSpec *spec, MobileSetti
                     NULL);
 
   heads = ms_head_tracker_get_heads (self->tracker);
-  for (int i = 0; i < heads->len; i++) {
+  for (guint i = 0; i < heads->len; i++) {
     MsHead *head = g_ptr_array_index (heads, i);
     g_debug ("Initial head: %s", head->name);
     on_head_added (self, head);
