@@ -224,8 +224,6 @@ test_setting_data_to_file_widget (PreferencesPageTestFixture *fixture, gconstpoi
 
   g_object_ref_sink (widget);
   g_object_unref (widget);
-  g_object_ref_sink (metadata->file_picker_label);
-  g_object_unref (metadata->file_picker_label);
   g_free (metadata);
 }
 
@@ -244,8 +242,6 @@ test_setting_data_to_file_widget_with_value (PreferencesPageTestFixture *fixture
 
   g_object_ref_sink (widget);
   g_object_unref (widget);
-  g_object_ref_sink (metadata->file_picker_label);
-  g_object_unref (metadata->file_picker_label);
   g_free (metadata);
 }
 
@@ -331,6 +327,22 @@ test_setting_data_to_number_widget_with_value (PreferencesPageTestFixture *fixtu
 }
 
 
+static void
+test_construct_empty_preferences_page (void)
+{
+  MsTweaksPage page = {
+    .weight = 999999,
+    .name = "Really heavy page",
+    .section_table = g_hash_table_new (g_str_hash, g_str_equal),
+    .name_i18n = "Riktigt tung sida",
+  };
+
+  g_assert_null (ms_tweaks_preferences_page_new (&page));
+
+  g_hash_table_unref (page.section_table);
+}
+
+
 int
 main (int argc, char *argv[])
 {
@@ -364,6 +376,8 @@ main (int argc, char *argv[])
                              test_setting_data_to_number_widget);
   PREFERENCES_PAGE_TEST_ADD ("/phosh-mobile-settings/test-tweaks-setting-data-to-number-widget-with-value",
                              test_setting_data_to_number_widget_with_value);
+  g_test_add_func ("/phosh-mobile-settings/test-construct-empty-preferences-page",
+                   test_construct_empty_preferences_page);
 
   return g_test_run ();
 }
